"use strict";
const core_1 = require("@angular/core");
const worker_1 = require("../../worker");
var STRING_SIMILARITY = require('string-similarity');
class CSVHandler {
    constructor() {
        this.headers = [];
        this.rows = [];
        this.guessTreshold = 0.7;
        this.headerModel = [];
        this.dataImported = new core_1.EventEmitter();
    }
    setRows(rows) {
        this.rows = rows;
    }
    formHeaders(headers) {
        let h = [];
        for (let head of headers) {
            if (head) {
                let matches = this.guessHeaders(head);
                let r = null;
                if (matches.bestMatch.rating > this.guessTreshold) {
                    r = matches.bestMatch.target;
                }
                h.push({
                    slug: head,
                    model: r
                });
            }
        }
        return h;
    }
    guessHeaders(header) {
        let stringHeader = [];
        for (let head of this.headerModel) {
            stringHeader.push(head.slug);
        }
        if (header) {
            return STRING_SIMILARITY.findBestMatch(header.toString(), stringHeader);
        }
    }
    setDataHeaders(dataHeader) {
        this.headerModel = dataHeader;
    }
    setHeaders(headers) {
        this.headers = this.formHeaders(headers);
    }
    exportData() {
        var worker = new worker_1.WorkerService();
        var data = {
            model: this.headers,
            rows: this.rows
        };
        //Make propers objects
        worker.work(data, function (data, done) {
            let rows = data.rows;
            let model = data.model;
            var newOBJArray = [];
            for (let row of rows) {
                var newRowFormat = {};
                for (let model_index in model) {
                    if (model[model_index].model) {
                        newRowFormat[model[model_index].model] = row[model_index];
                    }
                }
                if (Object.keys(newRowFormat).length !== 0) {
                    newOBJArray.push(newRowFormat);
                }
            }
            done(newOBJArray);
        }, (data) => {
            this.dataImported.emit(data.data);
        });
    }
    headerModelChanged(head) {
        var fil = this.headers.filter(function (item) {
            return head.model === item.model;
        });
        if (fil.length > 1) {
            fil.map((item) => {
                if (this.headers.indexOf(head) !== this.headers.indexOf(item)) {
                    item.model = null;
                }
                return item;
            });
        }
    }
    getRowsLenght() {
        return this.rows.length ? this.rows.length : 0;
    }
}
exports.CSVHandler = CSVHandler;
//# sourceMappingURL=csv-import.js.map