import {EventEmitter} from "@angular/core";
import {WorkerService} from "../../worker";
import {CSV_HEAD} from "../../../index";
var STRING_SIMILARITY = require('string-similarity');

export interface StringSimilarityMatch{
    bestMatch:Rating,
    ratings:Rating[]
}
export interface Rating{
    rating:number,
    target:string
}

export class CSVHandler {

    public headers = [];
    public rows = [];
    public guessTreshold = 0.7;


    public headerModel:CSV_HEAD[] = [];
    public dataImported = new EventEmitter();

    public setRows(rows) {
        this.rows = rows;
    }

    private formHeaders(headers) {
        let h = [];
        for (let head of headers) {
            if (head) {
                let matches = this.guessHeaders(head);

                let r = null;
                if (matches.bestMatch.rating > this.guessTreshold) {
                    r = matches.bestMatch.target;
                }
                h.push({
                    slug: head,
                    model: r
                });
            }
        }
        return h;
    }

    private guessHeaders(header):StringSimilarityMatch {


        let stringHeader:string[] = [];

        for(let head of this.headerModel){
            stringHeader.push(head.slug);
        }
        if (header){
            return STRING_SIMILARITY.findBestMatch(header.toString(), stringHeader);
        }
    }

    setDataHeaders(dataHeader:CSV_HEAD[]){
        this.headerModel = dataHeader;
    }

    setHeaders(headers) {
        this.headers = this.formHeaders(headers);
    }

    exportData() {
        var worker = new WorkerService();
        var data = {
            model: this.headers,
            rows: this.rows
        };

        //Make propers objects
        worker.work(data, function (data, done) {
            let rows = data.rows;
            let model = data.model;
            var newOBJArray = [];

            for (let row of rows) {
                var newRowFormat = {};

                for (let model_index in model) {
                    if (model[model_index].model) {
                        newRowFormat[model[model_index].model] = row[model_index];
                    }
                }
                if (Object.keys(newRowFormat).length !== 0) {
                    newOBJArray.push(newRowFormat);
                }

            }
            done(newOBJArray);
        }, (data)=> {
            this.dataImported.emit(data.data);
        });

    }

    headerModelChanged(head){

        var fil = this.headers.filter(function (item) {
            return head.model === item.model;
        });

        if(fil.length>1){
            fil.map((item)=>{
                if(this.headers.indexOf(head) !== this.headers.indexOf(item)){
                    item.model = null;
                }
                return item;
            });
        }
    }

    getRowsLenght(){
        return this.rows.length ? this.rows.length : 0;
    }

}
