"use strict";
const core_1 = require("@angular/core");
const clean_csv_1 = require("./works/clean-csv");
var Papa = require("papaparse");
var Encoding = require('encoding-japanese');
exports.PARSSING_STATES = {
    NO_ACTION: "No action",
    PROSSESING: "Processing data",
    DETECTING_ENCODING: "Detecting encoding",
    DONE: "Done",
    FAILED: "Failed"
};
exports.PARSING_ACTIONS = {
    AUTO_DETECT_ENCODING: "AUTO_DETECT"
};
class CSVParser {
    constructor() {
        this.workingState = exports.PARSSING_STATES.NO_ACTION;
        this.complete = new core_1.EventEmitter();
        this.failed = new core_1.EventEmitter();
        //Handling parsed papa data
        CSVParser.processing.subscribe((rows) => {
            this.workingState = exports.PARSSING_STATES.PROSSESING;
            this.handleRows(rows);
        });
        //Clean data
        CSVParser.parsingJobDone.subscribe((data) => {
            if (data) {
                this.workingState = exports.PARSSING_STATES.DONE;
                this.complete.emit(data);
            }
        });
        CSVParser.failedJob.subscribe((failed_reason) => {
            this.failed.emit(failed_reason);
            this.complete.emit({
                headers: [],
                rows: []
            });
        });
    }
    handleRows(rows) {
        clean_csv_1.cleanCSV(rows, (data) => {
            if (data) {
                CSVParser.parsingJobDone.emit(data.data);
            }
            else {
                this.workingState = exports.PARSSING_STATES.FAILED;
                CSVParser.failedJob.emit(data);
            }
        });
    }
    setFile(file) {
        this.file = file;
    }
    parse(encoding = exports.PARSING_ACTIONS.AUTO_DETECT_ENCODING, config = {}) {
        let ENCODING = "UTF-8";
        if (encoding === exports.PARSING_ACTIONS.AUTO_DETECT_ENCODING || encoding === null) {
            ENCODING = Encoding.detect(this.file);
            console.log(ENCODING);
        }
        let CONFIG = config;
        CONFIG['skipEmptyLines'] = true;
        CONFIG['encoding'] = ENCODING;
        CONFIG['complete'] = function (results) {
            CSVParser.onComplete(results);
        };
        Papa.parse(this.file, CONFIG);
    }
    static onComplete(rows) {
        CSVParser.processing.emit(rows);
    }
}
//EVENTS
CSVParser.processing = new core_1.EventEmitter();
CSVParser.parsingJobDone = new core_1.EventEmitter();
CSVParser.failedJob = new core_1.EventEmitter();
exports.CSVParser = CSVParser;
//# sourceMappingURL=csv-parse.js.map