import {EventEmitter} from '@angular/core';
import {cleanCSV} from "./works/clean-csv";

var Papa = require("papaparse");
var Encoding = require('encoding-japanese');

export interface PapaParseConfig{
    //http://papaparse.com/docs
    complete?: any,
    newline?:string,
    delimiter?:string,
    quoteChar?: string,
    header?: boolean,
    dynamicTyping?: boolean,
    preview?: number,
    encoding?: string,
    worker?: boolean,
    comments?: boolean,
    step?: any,
    error?:any,
    download?: boolean,
    skipEmptyLines?: boolean,
    chunk?: any,
    fastMode?: any,
    beforeFirstChunk?: any,
    withCredentials?: any
}

export const PARSSING_STATES = {

    NO_ACTION : "No action",
    PROSSESING: "Processing data",
    DETECTING_ENCODING: "Detecting encoding",
    DONE: "Done",
    FAILED: "Failed"

};

export const PARSING_ACTIONS = {
    AUTO_DETECT_ENCODING: "AUTO_DETECT"
};

export class CSVParser {

    file;
    workingState = PARSSING_STATES.NO_ACTION;
    //EVENTS
    private static processing = new EventEmitter();
    private static parsingJobDone = new EventEmitter();
    private static failedJob = new EventEmitter();

    public complete = new EventEmitter();
    public failed = new EventEmitter();

    constructor() {

        //Handling parsed papa data
        CSVParser.processing.subscribe((rows)=> {
            this.workingState = PARSSING_STATES.PROSSESING;
            this.handleRows(rows);
        });

        //Clean data
        CSVParser.parsingJobDone.subscribe((data)=> {
            if(data){
                this.workingState = PARSSING_STATES.DONE;
                this.complete.emit(data);
            }
        });

        CSVParser.failedJob.subscribe((failed_reason)=>{
            this.failed.emit(failed_reason);
            this.complete.emit({
                headers: [],
                rows:[]
            });
        });
    }

    handleRows(rows) {
        cleanCSV(rows,(data)=>{
            if(data){
                CSVParser.parsingJobDone.emit(data.data);
            }else {
                this.workingState = PARSSING_STATES.FAILED;
                CSVParser.failedJob.emit(data);
            }
        });
    }

    setFile(file) {
        this.file = file;
    }

    parse(encoding = PARSING_ACTIONS.AUTO_DETECT_ENCODING,config:PapaParseConfig = {}) {
        let ENCODING = "UTF-8";
        if(encoding === PARSING_ACTIONS.AUTO_DETECT_ENCODING || encoding === null){
            ENCODING = Encoding.detect(this.file);
            console.log(ENCODING);
        }

        let CONFIG = config;
        CONFIG['skipEmptyLines'] = true;
        CONFIG['encoding'] = ENCODING;
        CONFIG['complete'] = function (results) {
            CSVParser.onComplete(results);
        };

        Papa.parse(this.file,CONFIG);
    }

    public static onComplete(rows) {
        CSVParser.processing.emit(rows);
    }
}





