import {Injectable, NgZone} from "@angular/core";
import {CSVParser} from "./csv-service/csv-parse";
import {CSVHandler} from "./csv-service/csv-import";
import {CSVExport} from "./csv-service/csv-export";


@Injectable()
export class CSVService {

    //Headers for csv
    private dataHeaders;

    //parser parse csv data and return multidimensional array
    public parser: CSVParser = new CSVParser();
    //handler handling multidimensional array and make object array,
    //Object key set with this.handler.setDataHeaders
    public handler: CSVHandler = new CSVHandler();

    //Exporting csv file
    public exporter: CSVExport = new CSVExport();

    constructor(private zone: NgZone) {
        //EVENTS
        //Finished parsing and showing data
        this.onCSVParseCompleate();
        //Data ready
        this.onDataImport();
    }

    public parse(file,encoding = null) {
        //Set file
        this.parser.setFile(file);
        //Start parse
        this.parser.parse(encoding);
    }

    public setDataHeader(header:CSV_HEAD[]){
        this.dataHeaders = header;
    }

    private onCSVParseCompleate() {
        //Finished parsing and showing data
        this.parser.complete.subscribe((data)=> {
            this.zone.run(()=> {
                //Processing data and preparing to exportData
                this.handler.setRows(data.rows);
                this.handler.setDataHeaders(this.dataHeaders);
                this.handler.setHeaders(data.headers);
            });
        });
    }

    private onDataImport() {
        //EXPORT EVENT FIRED WHEN TRIGGERED  this.csv.handler.exportData();
        this.handler.dataImported.subscribe((data)=> {
            this.exporter.setJSON(data);

            //EXport data
            this.exporter.export();
        });
    }

}

export interface CSV_HEAD{
    slug:string,
    label:string,
    tr_label:string
}
